Code edit (10 edits merged)
Please save this source code
User prompt
Make a rotation matrix in the rotateTriangle function.
User prompt
Make a rotation matrix in the rotateTriangle function.
Code edit (3 edits merged)
Please save this source code
User prompt
optimise the code for speed
Code edit (1 edits merged)
Please save this source code
User prompt
Please fix the bug: 'setTimeout is not a function' in or related to this line: 'setTimeout(clearScreen, 3000);' Line Number: 174
Code edit (5 edits merged)
Please save this source code
User prompt
Make a translateTriangle function and centre the cube with the translateTriangle function.
Code edit (2 edits merged)
Please save this source code
User prompt
Please fix the bug: 'Cannot read properties of undefined (reading '0')' in or related to this line: 'var p1 = project(triangle[0], triangle[1], triangle[2]);' Line Number: 144
User prompt
In the projectTriangle function, use perspective project on the 3 points array in the triangle parameter, and return a 2 point array to draw.
Code edit (1 edits merged)
Please save this source code
Code edit (5 edits merged)
Please save this source code
User prompt
make an arg for wireframes on the drawTri function, default false.
Code edit (2 edits merged)
Please save this source code
User prompt
In the drawTri function, fill in the triangles using the scanline rendering algorithm.
User prompt
Fill in the triangles using the scanline rendering algorithm.
Code edit (2 edits merged)
Please save this source code
User prompt
Create a class "Cube" and store the triangles for now.
Code edit (1 edits merged)
Please save this source code
User prompt
optimise the code for speed
Code edit (2 edits merged)
Please save this source code
User prompt
Draw a 1000 random lines in a loop in between the start valuable and the console log
Code edit (1 edits merged)
Please save this source code
/****
* Initialize Game
****/
var game = new LK.Game({
backgroundColor: 0
});
/****
* Game Code
****/
//[x1, y1, x2, y2, x3, y3]
// Game constants for the grid
var GRID_SIZE = 64;
var PIXEL_SIZE = 2048 / GRID_SIZE;
// Cache children array reference
var screen = game.children;
function putPixel(x, y, color) {
if (x >= 0 && x < GRID_SIZE && y >= 0 && y < GRID_SIZE) {
screen[x + y * GRID_SIZE].tint = color;
}
}
// Line drawing function using Bresenham's line algorithm
function drawLine(x0, y0, x1, y1, color) {
var dx = x1 - x0;
var dy = y1 - y0;
var adx = dx < 0 ? -dx : dx;
var ady = dy < 0 ? -dy : dy;
var sx = dx < 0 ? -1 : 1;
var sy = dy < 0 ? -1 : 1;
var err = adx - ady;
var x = x0;
var y = y0;
while (true) {
putPixel(x, y, color);
if (x === x1 && y === y1) {
break;
}
var e2 = 2 * err;
if (e2 > -ady) {
err -= ady;
x += sx;
}
if (e2 < adx) {
err += adx;
y += sy;
}
}
}
function drawTri(tri, color, wireframe) {
if (wireframe === undefined) wireframe = false;
// Extract triangle vertices
var x1 = tri[0];
var y1 = tri[1];
var x2 = tri[2];
var y2 = tri[3];
var x3 = tri[4];
var y3 = tri[5];
// Sort vertices by y (ascending)
var verts = [{
x: x1,
y: y1
}, {
x: x2,
y: y2
}, {
x: x3,
y: y3
}];
verts.sort(function (a, b) {
return a.y - b.y;
});
var v0 = verts[0];
var v1 = verts[1];
var v2 = verts[2];
// Helper to interpolate x between two points at a given y
function interpX(y, x0, y0, x1, y1) {
if (y1 === y0) {
return x0;
}
return x0 + (x1 - x0) * (y - y0) / (y1 - y0);
}
// Fill bottom flat triangle (v0, v1, v2) if v1.y != v0.y
if (v1.y !== v0.y) {
for (var y = Math.ceil(v0.y); y < Math.ceil(v1.y); y++) {
var xA = interpX(y, v0.x, v0.y, v2.x, v2.y);
var xB = interpX(y, v0.x, v0.y, v1.x, v1.y);
var xStart = Math.ceil(Math.min(xA, xB));
var xEnd = Math.floor(Math.max(xA, xB));
for (var x = xStart; x <= xEnd; x++) {
putPixel(x, y, color);
}
}
}
// Fill top flat triangle (v1, v2, v0) if v2.y != v1.y
if (v2.y !== v1.y) {
for (var y = Math.ceil(v1.y); y < Math.ceil(v2.y); y++) {
var xA = interpX(y, v0.x, v0.y, v2.x, v2.y);
var xB = interpX(y, v1.x, v1.y, v2.x, v2.y);
var xStart = Math.ceil(Math.min(xA, xB));
var xEnd = Math.floor(Math.max(xA, xB));
for (var x = xStart; x <= xEnd; x++) {
putPixel(x, y, color);
}
}
}
// Draw the three sides of the triangle only if wireframe is true
if (wireframe) {
drawLine(x1, y1, x2, y2, color); // Side 1-2
drawLine(x2, y2, x3, y3, color); // Side 2-3
drawLine(x3, y3, x1, y1, color); // Side 3-1
}
}
var colorIndex = 0;
var offset = 2732 / 2 - GRID_SIZE * PIXEL_SIZE / 2;
// Create the grid of pixels
for (var y = 0; y < GRID_SIZE; y++) {
for (var x = 0; x < GRID_SIZE; x++) {
// Get an instance of our pixel asset
var pixel = LK.getAsset('pixel', {
x: x * PIXEL_SIZE,
y: y * PIXEL_SIZE + offset,
tint: 0
});
game.addChild(pixel);
}
}
//Start of the 3d renderer
var triangles = [[0, 0, 32, 0, 0, 32], [0, 32, 32, 0, 32, 32]];
var start = Date.now();
triangles.forEach(function (tri, i) {
drawTri(tri, 0xFFFFFF);
});
console.log("Time: " + String(Date.now() - start)); ===================================================================
--- original.js
+++ change.js
@@ -45,9 +45,10 @@
y += sy;
}
}
}
-function drawTri(tri, color) {
+function drawTri(tri, color, wireframe) {
+ if (wireframe === undefined) wireframe = false;
// Extract triangle vertices
var x1 = tri[0];
var y1 = tri[1];
var x2 = tri[2];
@@ -101,12 +102,14 @@
putPixel(x, y, color);
}
}
}
- // Draw the three sides of the triangle
- drawLine(x1, y1, x2, y2, color); // Side 1-2
- drawLine(x2, y2, x3, y3, color); // Side 2-3
- drawLine(x3, y3, x1, y1, color); // Side 3-1
+ // Draw the three sides of the triangle only if wireframe is true
+ if (wireframe) {
+ drawLine(x1, y1, x2, y2, color); // Side 1-2
+ drawLine(x2, y2, x3, y3, color); // Side 2-3
+ drawLine(x3, y3, x1, y1, color); // Side 3-1
+ }
}
var colorIndex = 0;
var offset = 2732 / 2 - GRID_SIZE * PIXEL_SIZE / 2;
// Create the grid of pixels
@@ -121,9 +124,9 @@
game.addChild(pixel);
}
}
//Start of the 3d renderer
-var triangles = [[0, 0, 8, 0, 0, 8], [0, 8, 8, 0, 8, 8]];
+var triangles = [[0, 0, 32, 0, 0, 32], [0, 32, 32, 0, 32, 32]];
var start = Date.now();
triangles.forEach(function (tri, i) {
drawTri(tri, 0xFFFFFF);
});